//+------------------------------------------------------------------+
//| Forex                                        20070921    |
//| Pivot Trader                                           |
//+------------------------------------------------------------------+
#include <stderror.mqh>
#include <stdlib.mqh>

// externs used for pivot trader 1
extern int pivot1TimeFrame = PERIOD_H1;
extern int pivot1PreviousOffset = 1;
extern double pivot1LimitFactor =0.5;   // calculate the limit as price + ( support - pivot ) * factor
extern double pivot1StopFactor = 17.0; 
extern int pivot1StopMin = 15;
extern int pivot1LimitMin = 15;
extern int pivot1TrendMAFast = 70;
extern int pivot1TrendMASlow = 75;
extern int pivot1MinEquity = 6000;
// externs used for pivot trader 2
extern int pivot2TimeFrame = PERIOD_H1;
extern int pivot2PreviousOffset = 1;
extern double pivot2LimitFactor =0.73;   // calculate the limit as price + ( support - pivot ) * factor
extern double pivot2StopFactor = 6.4; 
extern int pivot2StopMin = 15;
extern int pivot2LimitMin = 15;
extern int pivot2TrendMAFast = 20;
extern int pivot2TrendMASlow = 54;
extern int pivot2MinEquity = 30000;
// externs used for pivot trader 3
extern int pivot3TimeFrame = PERIOD_H1;
extern int pivot3PreviousOffset = 1;
extern double pivot3LimitFactor =0.75;   // calculate the limit as price + ( support - pivot ) * factor
extern double pivot3StopFactor = 11.5; 
extern int pivot3StopMin = 15;
extern int pivot3LimitMin = 15;
extern int pivot3TrendMAFast = 12;
extern int pivot3TrendMASlow = 29;
extern int pivot3MinEquity = 1;
// externs used for general config
extern double minLotSize = 0.1;
extern double maxLotSize = 0.1;
extern double lotIncrement = 0.1;
extern int approvedSlippage = 3;
extern int tradeOpenRetries = 2;
extern int enablePivotTrader1 = 1;
extern int enablePivotTrader2 = 1;
extern int enablePivotTrader3 = 1;
extern int minimumFreeMarginOnOrder = 100;
extern int gracePeriod = 18000;
// externs used in risk calcs
extern double riskExposure = 0.75;
extern int riskBuffer = 130;
extern double protectionFactor = 0.58;
extern double initialDeposit = 10000;

int magicPivotTrader1 = 27589;
int magicPivotTrader2 = 30918;
int magicPivotTrader3 = 46781;


//+------------------------------------------------------------------+
//| init()                                                           |
//+------------------------------------------------------------------+
int init() {
   
   // initialise randomness
   MathSrand( TimeLocal() );
     
}




//+------------------------------------------------------------------+
//| start()                                                          |
//+------------------------------------------------------------------+
int start() {
   
   double medianPrice;
   double triggerPoint;
   double accountStopOut;
   
   // if not sane, return
   if ( !do_sanity_check() ) return( 0 );
   
   do_check_trades();
      
}

//+------------------------------------------------------------------+
//| Main Check Code                                                  |
//+------------------------------------------------------------------+
int do_check_trades() {

	int liveOrders;	
	int loopOrders;
	int loopOrderMagic;
	bool flagPivotTrader1 = FALSE;
	bool flagPivotTrader2 = FALSE;
	bool flagPivotTrader3 = FALSE;	
	int pivotTrader1LifeTime, pivotTrader2LifeTime, pivotTrader3LifeTime;
   int returnValue;
      
   // first enumerate any live orders
	liveOrders = OrdersTotal();
	if ( OrdersTotal() > 0 ) {

		for ( loopOrders = 0; loopOrders < liveOrders; loopOrders ++ ) {

			if ( OrderSelect( loopOrders, SELECT_BY_POS ) ) {

				loopOrderMagic = OrderMagicNumber();
				if ( loopOrderMagic == magicPivotTrader1 ) {
				  flagPivotTrader1 = TRUE;
				  pivotTrader1LifeTime = TimeCurrent() - OrderOpenTime();
				}
				if ( loopOrderMagic == magicPivotTrader2 ) {
				  flagPivotTrader2 = TRUE;
				  pivotTrader2LifeTime = TimeCurrent() - OrderOpenTime();
            }               
				if ( loopOrderMagic == magicPivotTrader3 ) {
				  flagPivotTrader3 = TRUE;
				  pivotTrader3LifeTime = TimeCurrent() - OrderOpenTime();
            }               
         
			} else Print( StringConcatenate( "Execution error in do_check_trades(): ", GetLastError() ) );

		} 

	}

	// check pivot trader stuff
   if ( !flagPivotTrader3 && enablePivotTrader3 > 0 && AccountEquity() > pivot3MinEquity && ( pivotTrader1LifeTime == 0 || pivotTrader1LifeTime > gracePeriod ) && ( pivotTrader2LifeTime == 0 || pivotTrader2LifeTime > gracePeriod ) ) returnValue = do_check_pivot_trade3();
   if ( returnValue == 0 && !flagPivotTrader2 && enablePivotTrader2 > 0 && AccountEquity() > pivot2MinEquity && ( pivotTrader1LifeTime == 0 || pivotTrader1LifeTime > gracePeriod ) && ( pivotTrader3LifeTime == 0 || pivotTrader3LifeTime > gracePeriod ) ) returnValue = do_check_pivot_trade2();
   if ( returnValue == 0 && !flagPivotTrader1 && enablePivotTrader1 > 0 && AccountEquity() > pivot1MinEquity && ( pivotTrader2LifeTime == 0 || pivotTrader2LifeTime > gracePeriod ) && ( pivotTrader3LifeTime == 0 || pivotTrader3LifeTime > gracePeriod ) ) returnValue = do_check_pivot_trade1();

}





//+------------------------------------------------------------------+
//| Pivot Trader                                                     |
//+------------------------------------------------------------------+
int do_check_pivot_trade1() {

   double currentPivotPoint, currentSupport, currentResistance, currentPrice, oldPrice;
   double stopPrice, limitPrice;
   double currentMAFast, previousMAFast, currentMASlow, previousMASlow;
   int stopValue, limitValue;
   int returnValue;
   
   // calculate the pivot and prices   
   currentPivotPoint = ( iHigh( Symbol(), pivot1TimeFrame, pivot1PreviousOffset ) + iLow( Symbol(), pivot1TimeFrame, pivot1PreviousOffset ) + 2 * iClose( Symbol(), pivot1TimeFrame, pivot1PreviousOffset ) ) / 4;
   oldPrice = iClose( Symbol(), PERIOD_M1, 1 );  
   currentPrice = ( Ask + Bid ) / 2;   
   currentSupport = 2 * currentPivotPoint - iHigh( Symbol(), pivot1TimeFrame, pivot1PreviousOffset );
   currentResistance = 2 * currentPivotPoint - iLow( Symbol(), pivot1TimeFrame, pivot1PreviousOffset );
   currentMAFast = iMA( Symbol(), pivot1TimeFrame, pivot1TrendMAFast, 0, MODE_LWMA, PRICE_WEIGHTED, 0 );
   previousMAFast = iMA( Symbol(), pivot1TimeFrame, pivot1TrendMAFast, 0, MODE_LWMA, PRICE_WEIGHTED, 1 );
   currentMASlow = iMA( Symbol(), pivot1TimeFrame, pivot1TrendMASlow, 0, MODE_LWMA, PRICE_WEIGHTED, 0 );
   previousMASlow = iMA( Symbol(), pivot1TimeFrame, pivot1TrendMASlow, 0, MODE_LWMA, PRICE_WEIGHTED, 1 );
  
   // now test for a long trade
   if ( currentPrice > currentSupport && oldPrice < currentSupport && currentMAFast > previousMAFast && currentMASlow > previousMASlow && currentMAFast > currentMASlow ) {
      stopPrice = Ask - pivot1StopFactor * MathAbs( currentResistance - currentPivotPoint );
      limitPrice = Ask + pivot1LimitFactor * MathAbs( currentPivotPoint - currentSupport );
      stopValue = ( Ask - stopPrice ) / Point;
      limitValue = ( limitPrice - Ask ) / Point;
      if ( stopValue < pivot1StopMin || stopValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         stopPrice = Ask - MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot1StopMin ) * Point;
      }
      if ( limitValue < pivot1LimitMin || limitValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         limitPrice = Ask + MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot1LimitMin ) * Point;
      }      
      Print("pivot1 long");
      returnValue = do_open_long_order_pivot( magicPivotTrader1, stopPrice, limitPrice );
   }
   
   // now test for a short trade
   if ( currentPrice < currentResistance && oldPrice > currentResistance && currentMAFast < previousMAFast && currentMASlow < previousMASlow && currentMAFast < currentMASlow ) {
      stopPrice = Bid + pivot1StopFactor * MathAbs( currentResistance - currentPivotPoint );
      limitPrice = Bid - pivot1LimitFactor * MathAbs( currentPivotPoint - currentSupport );
      stopValue = ( stopPrice - Ask ) / Point;
      limitValue = ( Bid - limitPrice ) / Point;
      if ( stopValue < pivot1StopMin || stopValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         stopPrice = Bid + MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot1StopMin ) * Point;
      }
      if ( limitValue < pivot1LimitMin || limitValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         limitPrice = Bid - MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot1LimitMin ) * Point;
      }      
      Print("pivot1 short");
      returnValue = do_open_short_order_pivot( magicPivotTrader1, stopPrice, limitPrice );
   }   
     
   return( returnValue );  
     
}



int do_check_pivot_trade2() {

   double currentPivotPoint, currentSupport, currentResistance, currentPrice, oldPrice;
   double stopPrice, limitPrice;
   double currentMAFast, previousMAFast, currentMASlow, previousMASlow;
   int stopValue, limitValue;
   int returnValue;
   
   // calculate the pivot and prices   
   currentPivotPoint = ( iHigh( Symbol(), pivot2TimeFrame, pivot2PreviousOffset ) + iLow( Symbol(), pivot2TimeFrame, pivot2PreviousOffset ) + 2 * iClose( Symbol(), pivot2TimeFrame, pivot2PreviousOffset ) ) / 4;
   oldPrice = iClose( Symbol(), PERIOD_M1, 1 );  
   currentPrice = ( Ask + Bid ) / 2;
   currentSupport = currentPivotPoint - ( iHigh( Symbol(), pivot2TimeFrame, pivot2PreviousOffset ) - iLow( Symbol(), pivot2TimeFrame, pivot2PreviousOffset ) );
   currentResistance = currentPivotPoint + ( iHigh( Symbol(), pivot2TimeFrame, pivot2PreviousOffset ) - iLow( Symbol(), pivot2TimeFrame, pivot2PreviousOffset ) );
   currentMAFast = iMA( Symbol(), pivot2TimeFrame, pivot2TrendMAFast, 0, MODE_LWMA, PRICE_WEIGHTED, 0 );
   previousMAFast = iMA( Symbol(), pivot2TimeFrame, pivot2TrendMAFast, 0, MODE_LWMA, PRICE_WEIGHTED, 1 );
   currentMASlow = iMA( Symbol(), pivot2TimeFrame, pivot2TrendMASlow, 0, MODE_LWMA, PRICE_WEIGHTED, 0 );
   previousMASlow = iMA( Symbol(), pivot2TimeFrame, pivot2TrendMASlow, 0, MODE_LWMA, PRICE_WEIGHTED, 1 );
  
   // now test for a long trade
   if ( currentPrice > currentSupport && oldPrice < currentSupport && currentMAFast > previousMAFast && currentMASlow > previousMASlow && currentMAFast > currentMASlow ) {
      stopPrice = Ask - pivot2StopFactor * MathAbs( currentResistance - currentPivotPoint );
      limitPrice = Ask + pivot2LimitFactor * MathAbs( currentPivotPoint - currentSupport );
      stopValue = ( Ask - stopPrice ) / Point;
      limitValue = ( limitPrice - Ask ) / Point;
      if ( stopValue < pivot2StopMin || stopValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         stopPrice = Ask - MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot2StopMin ) * Point;
      }
      if ( limitValue < pivot2LimitMin || limitValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         limitPrice = Ask + MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot2LimitMin ) * Point;
      }      
      Print("pivot2 long");
      returnValue = do_open_long_order_pivot( magicPivotTrader2, stopPrice, limitPrice );
   }
   
   // now test for a short trade
   if ( currentPrice < currentResistance && oldPrice > currentResistance && currentMAFast < previousMAFast && currentMASlow < previousMASlow && currentMAFast < currentMASlow ) {
      stopPrice = Bid + pivot2StopFactor * MathAbs( currentResistance - currentPivotPoint );
      limitPrice = Bid - pivot2LimitFactor * MathAbs( currentPivotPoint - currentSupport );
      stopValue = ( stopPrice - Ask ) / Point;
      limitValue = ( Bid - limitPrice ) / Point;
      if ( stopValue < pivot2StopMin || stopValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         stopPrice = Bid + MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot2StopMin ) * Point;
      }
      if ( limitValue < pivot2LimitMin || limitValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         limitPrice = Bid - MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot2LimitMin ) * Point;
      }      
      Print("pivot2 short");
      returnValue = do_open_short_order_pivot( magicPivotTrader2, stopPrice, limitPrice );
   }   
     
   return( returnValue );
   
}


int do_check_pivot_trade3() {

   double currentPivotPoint, currentSupport, currentResistance, currentPrice, oldPrice;
   double stopPrice, limitPrice;
   double currentMAFast, previousMAFast, currentMASlow, previousMASlow;
   int stopValue, limitValue;
   int returnValue;
   
   // calculate the pivot and prices   
   currentPivotPoint = ( iHigh( Symbol(), pivot3TimeFrame, pivot3PreviousOffset ) + iLow( Symbol(), pivot3TimeFrame, pivot3PreviousOffset ) + 2 * iClose( Symbol(), pivot3TimeFrame, pivot3PreviousOffset ) ) / 4;
   oldPrice = iClose( Symbol(), PERIOD_M1, 1 );  
   currentPrice = ( Ask + Bid ) / 2;   
   currentSupport = 2 * currentPivotPoint - iHigh( Symbol(), pivot3TimeFrame, pivot3PreviousOffset );
   currentResistance = 2 * currentPivotPoint - iLow( Symbol(), pivot3TimeFrame, pivot3PreviousOffset );
   currentMAFast = iMA( Symbol(), pivot3TimeFrame, pivot3TrendMAFast, 0, MODE_LWMA, PRICE_WEIGHTED, 0 );
   previousMAFast = iMA( Symbol(), pivot3TimeFrame, pivot3TrendMAFast, 0, MODE_LWMA, PRICE_WEIGHTED, 1 );
   currentMASlow = iMA( Symbol(), pivot3TimeFrame, pivot3TrendMASlow, 0, MODE_LWMA, PRICE_WEIGHTED, 0 );
   previousMASlow = iMA( Symbol(), pivot3TimeFrame, pivot3TrendMASlow, 0, MODE_LWMA, PRICE_WEIGHTED, 1 );
  
   // now test for a long trade
   if ( currentPrice > currentSupport && oldPrice < currentSupport && currentMAFast > previousMAFast && currentMASlow > previousMASlow && currentMAFast > currentMASlow ) {
      stopPrice = Ask - pivot3StopFactor * MathAbs( currentResistance - currentPivotPoint );
      limitPrice = Ask + pivot3LimitFactor * MathAbs( currentPivotPoint - currentSupport );
      stopValue = ( Ask - stopPrice ) / Point;
      limitValue = ( limitPrice - Ask ) / Point;
      if ( stopValue < pivot3StopMin || stopValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         stopPrice = Ask - MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot3StopMin ) * Point;
      }
      if ( limitValue < pivot3LimitMin || limitValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         limitPrice = Ask + MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot3LimitMin ) * Point;
      }      
      Print("pivot3 long");
      returnValue = do_open_long_order_pivot( magicPivotTrader3, stopPrice, limitPrice );
   }
   
   // now test for a short trade
   if ( currentPrice < currentResistance && oldPrice > currentResistance && currentMAFast < previousMAFast && currentMASlow < previousMASlow && currentMAFast < currentMASlow ) {
      stopPrice = Bid + pivot3StopFactor * MathAbs( currentResistance - currentPivotPoint );
      limitPrice = Bid - pivot3LimitFactor * MathAbs( currentPivotPoint - currentSupport );
      stopValue = ( stopPrice - Ask ) / Point;
      limitValue = ( Bid - limitPrice ) / Point;
      if ( stopValue < pivot3StopMin || stopValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         stopPrice = Bid + MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot3StopMin ) * Point;
      }
      if ( limitValue < pivot3LimitMin || limitValue <= MarketInfo( Symbol(), MODE_STOPLEVEL ) ) {
         limitPrice = Bid - MathMax( MarketInfo( Symbol(), MODE_STOPLEVEL ), pivot3LimitMin ) * Point;
      }      
      Print("pivot3 short");
      returnValue = do_open_short_order_pivot( magicPivotTrader3, stopPrice, limitPrice );
   }   
   
   return( returnValue );
        
}




int do_open_long_order_pivot( int pivotMagic, double stopPrice, double limitPrice ) {

   double tradeLots;
   int returnValue;

   // first we have to work out how many lots we open with respect to risk
   tradeLots = do_calculate_risk( stopPrice, limitPrice );
   if ( tradeLots > 0 ) returnValue = do_open_generic_trade( Symbol(), OP_BUY, tradeLots, Ask, stopPrice, limitPrice, "pivot_trade_long", pivotMagic );
   return( returnValue );
}


int do_open_short_order_pivot( int pivotMagic, double stopPrice, double limitPrice ) {

   double tradeLots;
   int returnValue;

   // first we have to work out how many lots we open with respect to risk
   tradeLots = do_calculate_risk( stopPrice, limitPrice );
   if ( tradeLots > 0 ) returnValue = do_open_generic_trade( Symbol(), OP_SELL, tradeLots, Bid, stopPrice, limitPrice, "pivot_trade_short", pivotMagic );
   return( returnValue );

}



//+------------------------------------------------------------------+
//| Universal functions                                              |
//+------------------------------------------------------------------+
int do_open_generic_trade( string tradeSymbol, int tradeCMD, double tradeLots, double tradePrice, double tradeStop, double tradeLimit, string tradeComment, int tradeMagic ) {	

	int retryCounter = tradeOpenRetries;
	int openResult = 0;
	int errResult;
   int checkStop, checkLimit;

   // sanity check the trade
   if ( AccountFreeMarginCheck( tradeSymbol, tradeCMD, tradeLots ) <= minimumFreeMarginOnOrder || GetLastError() == ERR_NOT_ENOUGH_MONEY ) return(-1);
   if ( tradeCMD == OP_BUY ) {
      checkStop = ( Ask - tradeStop ) / Point;
      checkLimit = ( tradeLimit - Ask ) / Point;
   } else {
      checkStop = ( tradeStop - Bid ) / Point;
      checkLimit = ( Bid - tradeLimit ) / Point;
   }
   if ( checkStop < MarketInfo( tradeSymbol, MODE_STOPLEVEL ) || checkLimit < MarketInfo( tradeSymbol, MODE_STOPLEVEL ) ) return(-1);
   
   // do the trade
	openResult = OrderSend( tradeSymbol, tradeCMD, tradeLots, tradePrice, approvedSlippage, tradeStop, tradeLimit, tradeComment, tradeMagic, 0, Green );
   if ( openResult >= 0 ) Sleep(15000);  // after sending an order, make sure things settle a bit before doing anything else
	while ( openResult == -1 && retryCounter > 0 ) {

		// write out error to console
		errResult = GetLastError();
		Print( StringConcatenate( "Error opening trade on ", tradeSymbol, " to ", tradeCMD, " with ", tradeLots, "lots. Error: ", ErrorDescription(errResult) ) );
		
		// process error
		retryCounter--;
		if ( errResult == ERR_NO_ERROR ) retryCounter = 0; // eh??
		if ( errResult == ERR_INVALID_TRADE_PARAMETERS || errResult == ERR_COMMON_ERROR ) retryCounter = 0;  // its not looking good - just give up
		if ( errResult == ERR_SERVER_BUSY ) { 
			Sleep(15001); // we can't afford to wait several minutes, wait 15 secs try again, if not working give up
			retryCounter = 1;
		} 
		if ( errResult == ERR_NO_CONNECTION ) { 
			Sleep(15001); // again, can't wait that long, try 15 secs then if no go, give up
			if ( !IsConnected() ) { retryCounter = 0; } else retryCounter--;
		}
		if ( errResult == ERR_TOO_FREQUENT_REQUESTS ) {
			Sleep(10001); // uh-huh, ok i'll give it the standard 10 secs then try once more
			retryCounter = 1;
		}
		if ( errResult == ERR_ACCOUNT_DISABLED || errResult == ERR_INVALID_ACCOUNT ) retryCounter = 0; // not good, give up
		if ( errResult == ERR_TRADE_TIMEOUT ) retryCounter = 0; // we can't wait till things calm down to test, assume it worked - if not then not big problem
		if ( errResult == ERR_INVALID_PRICE ) {
			Sleep(10001); // we're not going by the rules, this may get tried several times
			RefreshRates();
		}
		if ( errResult == ERR_INVALID_STOPS || errResult == ERR_INVALID_STOPS ) {
			Sleep(10001);
			retryCounter = 1; // try once more
			RefreshRates();
		}
		if ( errResult == ERR_MARKET_CLOSED || errResult == ERR_TRADE_DISABLED || errResult == ERR_NOT_ENOUGH_MONEY ) retryCounter = 0; // give up		
		if ( errResult == ERR_PRICE_CHANGED || errResult == ERR_REQUOTE ) RefreshRates();
		if ( errResult == ERR_OFF_QUOTES ) {
			Sleep(5001);
			RefreshRates();
		}
		if ( errResult == ERR_ORDER_LOCKED ) {
			Sleep(15001);
			retryCounter = 0;  // who knows what happened....  just go on.
		}
		if ( errResult == ERR_LONG_POSITIONS_ONLY_ALLOWED ) retryCounter = 0; // WHATT!!!!
		if ( errResult == ERR_TOO_MANY_REQUESTS ) {
			Sleep(10001);
			RefreshRates();
		}
		if ( errResult == ERR_TRADE_CONTEXT_BUSY ) {
			if ( IsTradeContextBusy() ) Sleep(10001);
			RefreshRates();
		}
		if ( errResult == ERR_TRADE_TOO_MANY_ORDERS ) retryCounter = 0;

	}
   return(1);

}



double do_calculate_risk( int riskStop, int riskLimit ) {

   double tradeLots;
	double minTradeLotSize;
	double maxTradeLotSize;
	double lotTradeIncrement;
	double marginRequirementSingleLot;
	double contractSize;
	double accountLeverage;
	double tickValue;
	double accountFreeMargin;
   double tradingFunds;
	    
	// get min lot sizes
	minTradeLotSize = MarketInfo( Symbol(), MODE_MINLOT );
	if ( minTradeLotSize < 0 ) { Print("Invalid result in min lot size in do_calculate_risk()"); return(0.1); }
	minTradeLotSize = MathMax( minTradeLotSize, minLotSize );

	// get max lot sizes
	maxTradeLotSize = MarketInfo( Symbol(), MODE_MAXLOT );
	if ( maxTradeLotSize <= 0 ) { Print("Invalid result in max lot size in do_calculate_risk()"); return(0.1); }
	maxTradeLotSize = MathMin( maxTradeLotSize, maxLotSize );

	// get trade increment
	lotTradeIncrement = MarketInfo( Symbol(), MODE_LOTSTEP );
	if ( lotTradeIncrement <= 0 ) { Print("Invalid result in lotTradeIncrement in do_calculate_risk()"); return(0.1); }
	lotTradeIncrement = MathMax( lotTradeIncrement, lotIncrement );

	// get contract size
	contractSize = MarketInfo( Symbol(), MODE_LOTSIZE );
	if ( contractSize <= 0 ) { Print("Invalid result in contractsize in do_calculate_risk()"); return(0.1); }

	// get tick value (pipsize) in money for one lot
	tickValue = MarketInfo( Symbol(), MODE_TICKVALUE );
	if ( tickValue <= 0 ) { Print("Invalid result in tickvalue in do_calculate_risk()"); return(0.1); }

	// get account free margin
	accountFreeMargin = AccountFreeMargin();
	if ( accountFreeMargin <= 0 ) { Print("Invalid result in accountfreemargin in do_calculate_risk()"); return(0.1); }

	// get account leverage
	accountLeverage = AccountLeverage();
	if ( accountLeverage <= 0 ) { Print("Invalid result returned from accountLeverage() in do_calculate_risk()"); return(0.1); }

   // as the funds grow, we want to protect some of it
   if ( AccountFreeMargin() > initialDeposit ) {
      tradingFunds = AccountFreeMargin() * riskExposure - protectionFactor * ( AccountFreeMargin() - initialDeposit );
   } else {
      tradingFunds = AccountFreeMargin() * riskExposure;       
   }

	// do final calculation
   tradeLots = ( tradingFunds * AccountLeverage() ) / ( contractSize + AccountLeverage() * tickValue * ( riskStop + riskBuffer ) );
	tradeLots = NormalizeDouble( tradeLots / lotTradeIncrement, 0 ) * lotTradeIncrement;

	// check the results -- include riskProbabilityInclusionFactor
	if ( tradeLots < minTradeLotSize ) tradeLots = minTradeLotSize;
	if ( tradeLots > maxTradeLotSize ) tradeLots = maxTradeLotSize;

   return( tradeLots );

}



bool do_sanity_check() {

   if ( Bars < 200 ) return (FALSE);

   return( TRUE );
   
}